<?php

// Check if class already exists

//if (!class_exists("WPsite_Stats_FF")):

// AJAX functions for impression and click tracking

add_action( 'wp_ajax_wps_ff_record_impression', 			array('WPsite_Stats_FF', 'recordImpresssion'));
add_action( 'wp_ajax_nopriv_wps_ff_record_impression', 		array('WPsite_Stats_FF', 'recordImpresssion'));
add_action( 'wp_ajax_wps_ff_record_click', 					array('WPsite_Stats_FF', 'recordClick'));
add_action( 'wp_ajax_nopriv_wps_ff_record_click', 			array('WPsite_Stats_FF', 'recordClick'));

/**
 * WPsite_Stats_FF class.
 *
 * This class is used to record all stats for Flyout Fires.
 */
class WPsite_Stats_FF {

	/**
	 * table_name
	 *
	 * (default value: 'wpsite_flyout_fire_stats')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	public static $table_name = 'wpsite_flyout_fire_stats';

	/**
	 * Create Table when this class is loaded
	 *
	 * @access public
	 * @return void
	 */
	function __construct() {
		self::createTable();
	}

	/**
	 * Creates the Stats table if it does not exist
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function createTable(){

		global $wpdb;

		$wpdb->query("
			CREATE TABLE IF NOT EXISTS `" . self::getTableName() . "` (
				`id` int(11) NOT NULL AUTO_INCREMENT,
				`date` date NOT NULL,
				`flyout_fire_id` int(11) NOT NULL DEFAULT 0,
				`impressions` int(8) NOT NULL DEFAULT 0,
				`clicks` int(8) NOT NULL DEFAULT 0,
				PRIMARY KEY (`id`)
			) ENGINE=InnoDB DEFAULT CHARSET=latin1 AUTO_INCREMENT=1;
		");

	}

	/**
	 * Get the stats for all flyout_fires in a timeframe or a single firbar in a timeframe
	 *
	 * @access public
	 * @static
	 * @param mixed $start
	 * @param mixed $end
	 * @param mixed $id (default: null)
	 * @return void
	 */
	static function getStats($start, $end, $id = null, $select = '*') {

		// Safey to make sure the table is there
		self::createTable();

		$query = null;

		global $wpdb;

		// All Flyout Fires, All Time

		if ($start == null && $end == null && $id == null) {
			$query = 'SELECT ' . $select . ' FROM ' . self::getTableName();
		}

		// All Flyout Fires, After Date

		else if ($start != null && $end == null && $id == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` = %s', $start);
		}

		// All Flyout Fires, Date Range

		else if ($start != null && $end != null && $id == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `date` <= %s', $start, $end);
		}

		// Single Flyout Fire, All Time

		else if ($start == null && $end == null && $id != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `flyout_fire_id` = %d', $id);
		}

		// Single Flyout Fire, Date Range

		else if ($start != null && $end != null && $id != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `date` <= %s AND `flyout_fire_id` = %d', $start, $end, $id);
		}

		// No query was created

		if (!isset($query)) {
			return false;
		}

		$result = $wpdb->get_results($query, 'ARRAY_A');

		return $result;
	}

	/**
	 * Runs everyday and will add a record to table if none exists
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function initialize_flyout_fire_stats() {

		global $wpdb;
		$today = date(WPSITE_FLYOUT_FIRE_DATE_FORMAT);

		include_once(WPSITE_FLYOUT_FIRE_PLUGIN_DIR . "admin/db.php");

		$flyout_fires = $wpdb->get_results("SELECT * FROM `" . WPsite_Flyout_Fire_DB::getTableName() . "`");

		foreach ($flyout_fires as $flyout_fire) {

			// Check if entry already exsits

			$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `flyout_fire_id` = %d', $today, $flyout_fire->id);
			$flyout_fire_impressions = $wpdb->query($query);

			// Entry does not exist, create a new one and set impressions to 0 and clicks to 0

			if (!isset($flyout_fire_impressions) || $flyout_fire_impressions == 0) {

				$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
					`date`,
					`flyout_fire_id`,
					`impressions`,
					`clicks`
					) VALUES (%s, %d, 0, 0)',
					$today,
					$flyout_fire->id
				);

				$result = $wpdb->query($query);
			}
		}

		return $result;

	}

	/**
	 * Record an impression made by a flyout_fire
	 * This function is called via PHP.
	 *
	 * @access public
	 * @static
	 * @param mixed $data (default: null)
	 * @return void
	 */
	static function recordImpresssion($data = null) {

		$data['flyout_fire_id'] = $_POST['flyout_fire_id'];

		// Return false if no data is given

		if (!isset($data) || !isset($data['flyout_fire_id'])) {
			echo 'Flyout Fire impression was NOT able to be recored because the Flyout Fire ID was not retrieved.';
			die();
		}

		// Return false if crawler

		if (self::isBot()) {
			echo 'Flyout Fire impression was NOT able to be recored because a crawler was detected.';
			die();
		}

		// Return if user is admin or higher

		if (current_user_can('edit_published_posts')) {
			echo 'Flyout Fire impression was NOT able to be recored because current user is an admin';
			die();
		}

		global $wpdb;
		$today = date(WPSITE_FLYOUT_FIRE_DATE_FORMAT);

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `flyout_fire_id` = %d', $today, $data['flyout_fire_id']);
		$flyout_fire_impressions = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($flyout_fire_impressions) && $flyout_fire_impressions != 0) {

			$query = $wpdb->prepare('UPDATE ' . self::getTableName() . ' SET
				`impressions` = `impressions` + 1
				WHERE `date` = %s AND `flyout_fire_id` = %d', $today, $data['flyout_fire_id']);

			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set impressions to 1

		else {
			$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
				`date`,
				`flyout_fire_id`,
				`impressions`,
				`clicks`
				) VALUES (%s, %d, 1, 0)',
				$today,
				$data['flyout_fire_id']
			);

			$result = $wpdb->query($query);
		}

		if ($result) {
			echo 'Flyout Fire impression was able to be recored.';
		} else {
			echo 'Flyout Fire click was NOT able to be recored.';
		}

		die(); // necessary for WordPress AJAX calls

	}

	/**
	 * Record a click made by a flyout_fire
	 * This function is called via AJAX
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function recordClick() {

		$flyout_fire_id = $_POST['flyout_fire_id'];

		// Return if no data is given

		if (!isset($flyout_fire_id)) {
			echo 'Flyout Fire click was NOT able to be recored because the Flyout Fire ID was not retrieved.';
			die();
		}

		// Return if user is admin or higher

		if (current_user_can('edit_published_posts')) {
			echo 'Flyout Fire click was NOT able to be recored because current user is not authorized to log clicks.';
			die();
		}

		// Return false if crawler

		if (self::isBot()) {
			echo 'Flyout Fire click was NOT able to be recored because there is no user.  A crawler was detected';
			die();
		}

		global $wpdb;
		$today = date(WPSITE_FLYOUT_FIRE_DATE_FORMAT);

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `flyout_fire_id` = %d', $today, $flyout_fire_id);
		$flyout_fire_clicks = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($flyout_fire_clicks) && $flyout_fire_clicks != 0) {

			$query = $wpdb->prepare('UPDATE ' . self::getTableName() . ' SET
				`clicks` = `clicks` + 1
				WHERE `date` = %s AND `flyout_fire_id` = %d', $today, $flyout_fire_id);

			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set impressions to 1

		else {
			$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
				`date`,
				`flyout_fire_id`,
				`impressions`,
				`clicks`
				) VALUES (%s, %d, 0, 1)',
				$today,
				$flyout_fire_id
			);

			$result = $wpdb->query($query);
		}

		if ($result) {
			echo 'Flyout Fire click was able to be recored.';
		} else {
			echo 'Flyout Fire click was NOT able to be recored.';
		}

		die(); // necessary for WordPress AJAX calls
	}

	/**
	 * Deletes the stats for a flyout_fire when the flyout_fire is deleted
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function deleteStats($flyout_fire_id) {

		// Return false if flyout_fire_id is not set

		if (!isset($flyout_fire_id)) {
			return false;
		}

		global $wpdb;

		// Check if entry already exsits

		$query = $wpdb->prepare('DELETE FROM ' . self::getTableName() . ' WHERE `flyout_fire_id` = %d', $flyout_fire_id);
		$result = $wpdb->query($query);

		return $result;

	}

	/**
	 * Check for BOTs visiting site
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function isBot() {

		if (isset($_SERVER['HTTP_USER_AGENT']) && preg_match('/bot|spider|crawler|curl|^$/i', $_SERVER['HTTP_USER_AGENT'])) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Returns the proper table name based off of site ID
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function getTableName(){

		global $wpdb;

		return $wpdb->prefix . self::$table_name;
	}

}

//endif;